@@ -1,6 +1,7 @@
-utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"));
+utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType","ar.orders","i.orders","ma.orders"));
 
-auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c(1,frequency(data)),
+auto.ssarima <- function(data, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c(1,frequency(data)),
+                         combine=FALSE, workFast=TRUE,
                          initial=c("backcasting","optimal"), ic=c("AICc","AIC","BIC"),
                          cfType=c("MSE","MAE","HAM","MLSTFE","MSTFE","MSEh"),
                          h=10, holdout=FALSE,
@@ -19,6 +20,53 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
 # Add all the variables in ellipsis to current environment
     list2env(list(...),environment());
 
+    if(!is.null(orders)){
+        ar.max <- orders$ar;
+        i.max <- orders$i;
+        ma.max <- orders$ma;
+    }
+
+# If orders are provided in ellipsis via ar.max, write them down.
+    if(exists("ar.orders",inherits=FALSE)){
+        if(is.null(ar.orders)){
+            ar.max <- 0;
+        }
+        else{
+            ar.max <- ar.orders;
+        }
+    }
+    else{
+        if(is.null(orders)){
+            ar.max <- 0;
+        }
+    }
+    if(exists("i.orders",inherits=FALSE)){
+        if(is.null(i.orders)){
+            i.max <- 0;
+        }
+        else{
+            i.max <- i.orders;
+        }
+    }
+    else{
+        if(is.null(orders)){
+            i.max <- 0;
+        }
+    }
+    if(exists("ma.orders",inherits=FALSE)){
+        if(is.null(ma.orders)){
+            ma.max <- 0;
+        }
+        else{
+            ma.max <- ma.orders
+        }
+    }
+    else{
+        if(is.null(orders)){
+            ma.max <- 0;
+        }
+    }
+
 ##### Set environment for ssInput and make all the checks #####
     environment(ssAutoInput) <- environment();
     ssAutoInput(modelType="ssarima",ParentEnvironment=environment());
@@ -98,7 +146,7 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
 # Don't try to fix anything if the number of seasonalities is greater than 2
     if(length(lags)<=2){
         if(obsInsample <= n.param.max){
-            arma.length <- length(ar.max)
+            arma.length <- length(ar.max);
             while(obsInsample <= n.param.max){
                 if(any(c(ar.max[arma.length],ma.max[arma.length])>0)){
                     ar.max[arma.length] <- max(0,ar.max[arma.length] - 1);
@@ -151,13 +199,23 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
 # 1 stands for constant/no constant, another one stands for ARIMA(0,0,0)
     nModels <- prod(i.max + 1) * (1 + sum(ma.max*(1 + sum(ar.max)))) + 1;
     testModel <- list(NA);
+# Array with elements x maxorders x horizon x point/lower/upper
+    if(combine){
+        testForecasts <- list(NA);
+        testFitted <- list(NA);
+        testICs <- list(NA);
+        testLevels <- list(NA);
+        testStates <- list(NA);
+        testTransition <- list(NA);
+        testPersistence <- list(NA);
+    }
     ICValue <- 1E+100;
     m <- 0;
-    obsInsample <- length(data);
     constant <- TRUE;
 
     test.lags <- ma.test <- ar.test <- i.test <- rep(0,length(lags));
     ar.best <- ma.best <- i.best <- rep(0,length(lags));
+    ar.best.local <- ma.best.local <- i.best.local <- ar.best;
 
 #### Function corrects IC taking number of parameters on previous step ####
     icCorrector <- function(icValue, nParam, obsInsample, nParamNew){
@@ -184,7 +242,7 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
 ### If for some reason we have model with zeroes for orders, return it.
     if(all(c(ar.max,i.max,ma.max)==0)){
         cat("\b\b\b\bDone!\n");
-        bestModel <- ssarima(data,ar.orders=(ar.best),i.orders=(i.best),ma.orders=(ma.best),lags=(lags),
+        bestModel <- ssarima(data,orders=list(ar=ar.best,i=(i.best),ma=(ma.best)),lags=(lags),
                              constant=TRUE,initial=initialType,cfType=cfType,
                              h=h,holdout=holdout,
                              intervals=intervals,level=level,
@@ -216,21 +274,35 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
                 cat(paste0(rep("\b",nchar(round(m/nModels,2)*100)+1),collapse=""));
                 cat(paste0(round((m)/nModels,2)*100,"%"));
             }
-# cat("I: ");cat(i.orders[d,]);
             nParamOriginal <- 1;
-            testModel <- ssarima(data,ar.orders=0,i.orders=i.orders[d,],ma.orders=0,lags=lags,
+            testModel <- ssarima(data,orders=list(ar=0,i=i.orders[d,],ma=0),lags=lags,
                                  constant=TRUE,initial=initialType,cfType=cfType,
                                  h=h,holdout=holdout,
                                  intervals=intervals,level=level,
                                  intermittent=intermittent,silent=TRUE,
                                  xreg=xreg,updateX=updateX,FI=FI);
             ICValue <- testModel$ICs[ic];
-# print(ICValue);
+            if(combine){
+                testForecasts[[m]] <- matrix(NA,h,3);
+                testForecasts[[m]][,1] <- testModel$forecast;
+                testForecasts[[m]][,2] <- testModel$lower;
+                testForecasts[[m]][,3] <- testModel$upper;
+                testFitted[[m]] <- testModel$fitted;
+                testICs[[m]] <- ICValue;
+                testLevels[[m]] <- 1;
+                testStates[[m]] <- testModel$states;
+                testTransition[[m]] <- testModel$transition;
+                testPersistence[[m]] <- testModel$persistence;
+            }
+            if(silent[1]=="d"){
+                cat("I: ");cat(i.orders[d,]);cat(", ");
+                cat(ICValue); cat("\n");
+            }
             if(m==1){
                 bestIC <- ICValue;
                 dataI <- testModel$residuals;
                 i.best <- i.orders[d,];
-                bestICI <- bestICMA <- bestIC;
+                bestICAR <- bestICI <- bestICMA <- bestIC;
             }
             else{
                 if(ICValue < bestICI){
@@ -243,14 +315,17 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
                     }
                 }
                 else{
-                    m <- m + sum(ma.max*(1 + sum(ar.max)));
-                    next;
+                    if(workFast){
+                        m <- m + sum(ma.max*(1 + sum(ar.max)));
+                        next;
+                    }
                 }
             }
 
 ##### Loop for MA #####
             if(any(ma.max!=0)){
-                ma.test <- rep(0,length(ma.test));
+                bestICMA <- bestICI;
+                ma.best.local <- ma.test <- rep(0,length(ma.test));
                 for(seasSelectMA in 1:length(lags)){
                     if(ma.max[seasSelectMA]!=0){
                         for(maSelect in 1:ma.max[seasSelectMA]){
@@ -260,20 +335,35 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
                                 cat(paste0(round((m)/nModels,2)*100,"%"));
                             }
                             ma.test[seasSelectMA] <- ma.max[seasSelectMA] - maSelect + 1;
-# cat("MA: ");cat(ma.test);
                             nParamMA <- sum(ma.test);
                             nParamNew <- nParamOriginal + nParamMA;
 
-                            testModel <- ssarima(dataI,ar.orders=0,i.orders=0,ma.orders=ma.test,lags=lags,
+                            testModel <- ssarima(dataI,orders=list(ar=0,i=0,ma=ma.test),lags=lags,
                                                  constant=FALSE,initial=initialType,cfType=cfType,
                                                  h=h,holdout=FALSE,
                                                  intervals=intervals,level=level,
                                                  intermittent=intermittent,silent=TRUE,
                                                  xreg=xreg,updateX=updateX,FI=FI);
                             ICValue <- icCorrector(testModel$ICs[ic], nParamMA, obsInsample, nParamNew);
-# print(ICValue);
+                            if(combine){
+                                testForecasts[[m]] <- matrix(NA,h,3);
+                                testForecasts[[m]][,1] <- testModel$forecast;
+                                testForecasts[[m]][,2] <- testModel$lower;
+                                testForecasts[[m]][,3] <- testModel$upper;
+                                testFitted[[m]] <- testModel$fitted;
+                                testICs[[m]] <- ICValue;
+                                testLevels[[m]] <- 2;
+                                testStates[[m]] <- testModel$states;
+                                testTransition[[m]] <- testModel$transition;
+                                testPersistence[[m]] <- testModel$persistence;
+                            }
+                            if(silent[1]=="d"){
+                                cat("MA: ");cat(ma.test);cat(", ");
+                                cat(ICValue); cat("\n");
+                            }
                             if(ICValue < bestICMA){
                                 bestICMA <- ICValue;
+                                ma.best.local <- ma.test;
                                 if(ICValue < bestIC){
                                     bestIC <- bestICMA;
                                     i.best <- i.orders[d,];
@@ -283,14 +373,20 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
                                 dataMA <- testModel$residuals;
                             }
                             else{
-                                m <- m + ma.test[seasSelectMA] * (1 + sum(ar.max)) - 1;
-                                ma.test <- ma.best;
-                                break;
+                                if(workFast){
+                                    m <- m + ma.test[seasSelectMA] * (1 + sum(ar.max)) - 1;
+                                    ma.test <- ma.best.local;
+                                    break;
+                                }
+                                else{
+                                    ma.test <- ma.best.local;
+                                }
                             }
 
 ##### Loop for AR #####
                             if(any(ar.max!=0)){
-                                ar.test <- rep(0,length(ar.test));
+                                bestICAR <- bestICMA;
+                                ar.best.local <- ar.test <- rep(0,length(ar.test));
                                 for(seasSelectAR in 1:length(lags)){
                                     test.lags[seasSelectAR] <- lags[seasSelectAR];
                                     if(ar.max[seasSelectAR]!=0){
@@ -301,28 +397,51 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
                                                 cat(paste0(round((m)/nModels,2)*100,"%"));
                                             }
                                             ar.test[seasSelectAR] <- ar.max[seasSelectAR] - arSelect + 1;
-# cat("AR: ");cat(ar.test);
                                             nParamAR <- sum(ar.test);
                                             nParamNew <- nParamOriginal + nParamMA + nParamAR;
 
-                                            testModel <- ssarima(dataMA,ar.orders=ar.test,i.orders=0,ma.orders=0,lags=lags,
+                                            testModel <- ssarima(dataMA,orders=list(ar=ar.test,i=0,ma=0),lags=lags,
                                                                  constant=FALSE,initial=initialType,cfType=cfType,
                                                                  h=h,holdout=FALSE,
                                                                  intervals=intervals,level=level,
                                                                  intermittent=intermittent,silent=TRUE,
                                                                  xreg=xreg,updateX=updateX,FI=FI);
                                             ICValue <- icCorrector(testModel$ICs[ic], nParamAR, obsInsample, nParamNew);
-# print(ICValue);
-                                            if(ICValue < bestIC){
-                                                bestIC <- ICValue;
-                                                i.best <- i.orders[d,];
-                                                ar.best <- ar.test;
-                                                ma.best <- ma.test;
+                                            if(combine){
+                                                testForecasts[[m]] <- matrix(NA,h,3);
+                                                testForecasts[[m]][,1] <- testModel$forecast;
+                                                testForecasts[[m]][,2] <- testModel$lower;
+                                                testForecasts[[m]][,3] <- testModel$upper;
+                                                testFitted[[m]] <- testModel$fitted;
+                                                testICs[[m]] <- ICValue;
+                                                testLevels[[m]] <- 3;
+                                                testStates[[m]] <- testModel$states;
+                                                testTransition[[m]] <- testModel$transition;
+                                                testPersistence[[m]] <- testModel$persistence;
+                                            }
+                                            if(silent[1]=="d"){
+                                                cat("AR: ");cat(ar.test);cat(", ");
+                                                cat(ICValue); cat("\n");
+                                            }
+                                            if(ICValue < bestICAR){
+                                                bestICAR <- ICValue;
+                                                ar.best.local <- ar.test;
+                                                if(ICValue < bestIC){
+                                                    bestIC <- ICValue;
+                                                    i.best <- i.orders[d,];
+                                                    ar.best <- ar.test;
+                                                    ma.best <- ma.test;
+                                                }
                                             }
                                             else{
-                                                m <- m + ar.test[seasSelectAR] - 1;
-                                                ar.test <- ar.best;
-                                                break;
+                                                if(workFast){
+                                                    m <- m + ar.test[seasSelectAR] - 1;
+                                                    ar.test <- ar.best.local;
+                                                    break;
+                                                }
+                                                else{
+                                                    ar.test <- ar.best.local;
+                                                }
                                             }
                                         }
                                     }
@@ -343,13 +462,25 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
 
 #### Test the constant ####
     if(any(c(ar.best,i.best,ma.best)!=0)){
-        testModel <- ssarima(data,ar.orders=(ar.best),i.orders=(i.best),ma.orders=(ma.best),lags=(lags),
+        testModel <- ssarima(data,orders=list(ar=(ar.best),i=(i.best),ma=(ma.best)),lags=(lags),
                              constant=FALSE,initial=initialType,cfType=cfType,
                              h=h,holdout=holdout,
                              intervals=intervals,level=level,
                              intermittent=intermittent,silent=TRUE,
                              xreg=xreg,updateX=updateX,FI=FI);
         ICValue <- testModel$ICs[ic];
+        if(combine){
+            testForecasts[[m]] <- matrix(NA,h,3);
+            testForecasts[[m]][,1] <- testModel$forecast;
+            testForecasts[[m]][,2] <- testModel$lower;
+            testForecasts[[m]][,3] <- testModel$upper;
+            testFitted[[m]] <- testModel$fitted;
+            testICs[[m]] <- ICValue;
+            testLevels[[m]] <- 1;
+            testStates[[m]] <- testModel$states;
+            testTransition[[m]] <- testModel$transition;
+            testPersistence[[m]] <- testModel$persistence;
+        }
 # cat("Constant: ");print(ICValue);
         if(ICValue < bestIC){
             bestModel <- testModel;
@@ -360,26 +491,76 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
         }
     }
 
-    if(constant){
-#### Reestimate the best model in order to get rid of bias ####
-        bestModel <- ssarima(data,ar.orders=(ar.best),i.orders=(i.best),ma.orders=(ma.best),lags=(lags),
-                             constant=TRUE,initial=initialType,cfType=cfType,
-                             h=h,holdout=holdout,
-                             intervals=intervals,level=level,
-                             intermittent=intermittent,silent=TRUE,
-                             xreg=xreg,updateX=updateX,FI=FI);
+    if(combine){
+        testICs <- unlist(testICs);
+        testLevels <- unlist(testLevels);
+        testForecasts <- array(unlist(testForecasts),c(h,3,length(testICs)));
+        testFitted <- matrix(unlist(testFitted),ncol=length(testICs));
+        icWeights <- exp(-0.5*(testICs-min(testICs)))/sum(exp(-0.5*(testICs-min(testICs))));
+
+        testForecastsNew <- testForecasts;
+        testFittedNew <- testFitted;
+        for(i in 1:length(testLevels)){
+            if(testLevels[i]==1){
+                j <- i;
+            }
+            else if(testLevels[i]==2){
+                k <- i;
+                testForecastsNew[,,i] <- testForecasts[,,j] + testForecasts[,,i];
+                testFittedNew[,i] <- testFitted[,j] + testFitted[,i];
+            }
+            else if(testLevels[i]==3){
+                testForecastsNew[,,i] <- testForecasts[,,j] + testForecasts[,,k] + testForecasts[,,i];
+                testFittedNew[,i] <- testFitted[,j] + testFitted[,k] + testFitted[,i];
+            }
+        }
+        y.for <- ts(testForecastsNew[,1,] %*% icWeights,start=start(testModel$forecast),frequency=frequency(testModel$forecast));
+        y.low <- ts(testForecastsNew[,2,] %*% icWeights,start=start(testModel$lower),frequency=frequency(testModel$lower));
+        y.high <- ts(testForecastsNew[,3,] %*% icWeights,start=start(testModel$upper),frequency=frequency(testModel$upper));
+        y.fit <- ts(testFittedNew %*% icWeights,start=start(testModel$fitted),frequency=frequency(testModel$fitted));
+        modelname <- "ARIMA combined";
+
+        errors <- ts(y-c(y.fit),start=start(y.fit),frequency=frequency(y.fit));
+        y.holdout <- ts(data[(obsInsample+1):obsAll],start=start(testModel$forecast),frequency=frequency(testModel$forecast));
+        s2 <- mean(errors^2);
+        errormeasures <- errorMeasurer(y.holdout,y.for,y);
+        ICs <- c(t(testICs) %*% icWeights);
+        names(ICs) <- ic;
+
+        bestModel <- list(model=modelname,timeElapsed=Sys.time()-startTime,
+                          initialType=initialType,
+                          fitted=y.fit,forecast=y.for,
+                          lower=y.low,upper=y.high,residuals=errors,s2=s2,intervals=intervals,level=level,
+                          actuals=data,holdout=y.holdout,intermittent=intermittent,
+                          xreg=xreg,updateX=updateX,
+                          ICs=ICs,ICw=icWeights,cf=NULL,cfType=cfType,accuracy=errormeasures);
+
+        bestModel <- structure(bestModel,class="smooth");
+    }
+    else{
+        if(constant){
+            #### Reestimate the best model in order to get rid of bias ####
+            bestModel <- ssarima(data,orders=list(ar=(ar.best),i=(i.best),ma=(ma.best)),lags=(lags),
+                                 constant=TRUE,initial=initialType,cfType=cfType,
+                                 h=h,holdout=holdout,
+                                 intervals=intervals,level=level,
+                                 intermittent=intermittent,silent=TRUE,
+                                 xreg=xreg,updateX=updateX,FI=FI);
+        }
+
+        y.fit <- bestModel$fitted;
+        y.for <- bestModel$forecast;
+        y.high <- bestModel$upper;
+        y.low <- bestModel$lower;
+        modelname <- bestModel$model;
+
+        bestModel$timeElapsed <- Sys.time()-startTime;
     }
 
     if(silentText==FALSE){
         cat("... Done! \n");
     }
 
-    y.fit <- bestModel$fitted;
-    y.for <- bestModel$forecast;
-    y.high <- bestModel$upper;
-    y.low <- bestModel$lower;
-    modelname <- bestModel$model;
-
     # Make plot
     if(silentGraph==FALSE){
         if(intervals==TRUE){
@@ -392,6 +573,5 @@ auto.ssarima <- function(data,ar.max=c(3,3), i.max=c(2,1), ma.max=c(3,3), lags=c
         }
     }
 
-    bestModel$timeElapsed <- Sys.time()-startTime;
     return(bestModel);
 }
