@@ -132,7 +132,8 @@ ces <- function(data, seasonality=c("none","simple","partial","full"),
     Stype <- "N";
 
 # Check number of parameters vs data
-    n.param.max <- n.param.max + FXEstimate*length(matFX) + gXEstimate*nrow(vecgX) + initialXEstimate*ncol(matat);
+    n.param.exo <- FXEstimate*length(matFX) + gXEstimate*nrow(vecgX) + initialXEstimate*ncol(matat);
+    n.param.max <- n.param.max + n.param.exo + (intermittent!="n");
 
 ##### Check number of observations vs number of max parameters #####
     if(obsNonzero <= n.param.max){
@@ -428,10 +429,6 @@ CreatorCES <- function(silentText=FALSE,...){
     ssForecaster(ParentEnvironment=environment());
 
 ##### Do final check and make some preparations for output #####
-    if(any(is.na(y.fit),is.na(y.for))){
-        message("Something went wrong during the optimisation and NAs were produced!");
-        message("Please check the input and report this error if it persists to the maintainer.");
-    }
 
 # Write down initials of states vector and exogenous
     if(initialType!="p"){
